@@ -1,23 +1,21 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-"""
-Primitivus: a SAT frontend
-Copyright (C) 2009, 2010  Jérôme Poisson (goffi@goffi.org)
-
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# Urwid SàT extensions
+# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import urwid
 from logging import debug, info, warning, error
@@ -43,7 +41,7 @@ class AdvancedEdit(urwid.Edit):
                     - the text to complete
                     - if there was already a completion, a dict with
                         - 'completed':last completion
-                        - 'completion_pos': cursor position where the completion starts 
+                        - 'completion_pos': cursor position where the completion starts
                         - 'position': last completion cursor position
                       this dict must be used (and can be filled) to find next completion)
                    and which return the full text completed"""
@@ -59,7 +57,7 @@ class AdvancedEdit(urwid.Edit):
         elif key == 'ctrl k':
             self._delete_highlighted()
             self.set_edit_text(self.edit_text[:self.edit_pos])
-        elif key == 'ctrl w':    
+        elif key == 'ctrl w':
             before = self.edit_text[:self.edit_pos]
             pos = before.rstrip().rfind(" ")+1
             self.set_edit_text(before[:pos] + self.edit_text[self.edit_pos:])
@@ -85,7 +83,8 @@ class AdvancedEdit(urwid.Edit):
             except AttributeError:
                 #No completion method defined
                 pass
-        return super(AdvancedEdit, self).keypress(size, key) 
+        return super(AdvancedEdit, self).keypress(size, key)
+
 
 class Password(AdvancedEdit):
     """Edit box which doesn't show what is entered (show '*' or other char instead)"""
@@ -112,10 +111,11 @@ class Password(AdvancedEdit):
 
     def render(self, size, focus=False):
         return super(Password, self).render(size, focus)
-       
+
+
 class ModalEdit(AdvancedEdit):
     """AdvancedEdit with vi-like mode management
-    - there is a new 'mode' property wich can be changed with properties
+    - there is a new 'mode' property which can be changed with properties
     specified during init
     - completion callback received a new 'mode' argument
     """
@@ -142,7 +142,7 @@ class ModalEdit(AdvancedEdit):
             if self._modes[key][0] == value:
                 mode_key = key
                 break
-        
+
         mode, caption = self._modes[mode_key]
         self._mode = mode
         self.set_caption(caption)
@@ -160,10 +160,12 @@ class ModalEdit(AdvancedEdit):
         if self._mode == 'NORMAL' and key in self._modes:
             self.mode = self._modes[key][0]
             return
-        return super(ModalEdit, self).keypress(size, key) 
+        return super(ModalEdit, self).keypress(size, key)
 
-class SurroundedText(urwid.FlowWidget):
+
+class SurroundedText(urwid.Widget):
     """Text centered on a repeated character (like a Divider, but with a text in the center)"""
+    _sizing = frozenset(['flow'])
 
     def __init__(self,text,car=utf8decode('─')):
         self.text=text
@@ -181,10 +183,11 @@ class SurroundedText(urwid.FlowWidget):
         render_text = middle * self.car + self.text + (maxcol - len(self.text) - middle) * self.car
         return urwid.Text(render_text)
 
+
 class SelectableText(urwid.WidgetWrap):
     """Text which can be selected with space"""
     signals = ['change']
-    
+
     def __init__(self, text, align='left', header='', focus_attr='default_focus', selected_text=None, selected=False, data=None):
         """@param text: same as urwid.Text's text parameter
         @param align: same as urwid.Text's align parameter
@@ -193,29 +196,12 @@ class SelectableText(urwid.WidgetWrap):
         self.focus_attr = focus_attr
         self.__selected = False
         self.__was_focused = False
-        self.header = self.__valid_text(header)
-        self.default_txt = self.__valid_text(text)
+        self.header = header
+        self.default_txt = text
         urwid.WidgetWrap.__init__(self, urwid.Text("",align=align))
         self.setSelectedText(selected_text)
         self.setState(selected)
 
-    def __valid_text(self, text):
-        """Tmp method needed until dbus and urwid are more friends"""
-        if isinstance(text,basestring):
-            return unicode(text)
-        elif isinstance(text,tuple):
-            return (unicode(text[0]),text[1])
-        elif isinstance(text,list):
-            for idx in range(len(text)):
-                elem = text[idx]
-                if isinstance(elem,basestring):
-                    text[idx] = unicode(elem)
-                if isinstance(elem,tuple):
-                    text[idx] = (unicode(elem[0]),elem[1])
-        else:
-            warning (_('WARNING: unknown text type'))
-        return text
-
     def getValue(self):
         if isinstance(self.default_txt,basestring):
             return self.default_txt
@@ -234,7 +220,7 @@ class SelectableText(urwid.WidgetWrap):
 
     def set_text(self, text):
         """/!\ set_text doesn't change self.selected_txt !"""
-        self.default_txt = self.__valid_text(text)
+        self.default_txt = text
         self.setState(self.__selected,invisible=True)
 
     def setSelectedText(self, text=None):
@@ -242,7 +228,7 @@ class SelectableText(urwid.WidgetWrap):
         @text: text as in urwid.Text or None for default value"""
         if text == None:
             text = ('selected',self.getValue())
-        self.selected_txt = self.__valid_text(text)
+        self.selected_txt = text
         if self.__selected:
             self.setState(self.__selected)
 
@@ -256,7 +242,7 @@ class SelectableText(urwid.WidgetWrap):
             txt_list.append(txt)
         self._w.base_widget.set_text(txt_list)
 
-    
+
     def setState(self, selected, invisible=False):
         """Change state
         @param selected: boolean state value
@@ -268,7 +254,7 @@ class SelectableText(urwid.WidgetWrap):
         self._invalidate()
         if not invisible:
             self._emit("change", self.__selected)
-   
+
     def getState(self):
         return self.__selected
 
@@ -285,9 +271,9 @@ class SelectableText(urwid.WidgetWrap):
         if is_mouse_press(event) and button == 1:
             self.setState(not self.__selected)
             return True
-        
+
         return False
-   
+
     def render(self, size, focus=False):
         attr_list = self._w.base_widget._attrib
         if not focus:
@@ -312,21 +298,23 @@ class SelectableText(urwid.WidgetWrap):
                 self.__was_focused = True #bloody ugly hack :)
         return self._w.render(size, focus)
 
+
 class ClickableText(SelectableText):
     signals = SelectableText.signals + ['click']
-    
+
     def setState(self, selected, invisible=False):
         super(ClickableText,self).setState(False,True)
         if not invisible:
             self._emit('click')
 
+
 class CustomButton(ClickableText):
 
     def __init__(self, label, on_press=None, user_data=None, left_border = "[ ", right_border = " ]"):
         self.label = label
         self.left_border = left_border
         self.right_border = right_border
-        super(CustomButton, self).__init__([left_border, label, right_border]) 
+        super(CustomButton, self).__init__([left_border, label, right_border])
         self.size = len(self.get_text())
         if on_press:
             urwid.connect_signal(self, 'click', on_press, user_data)
@@ -342,34 +330,95 @@ class CustomButton(ClickableText):
         self.label = label
         self.set_text([self.left_border, label, self.right_border])
 
+
+class ListOption(unicode):
+    """ Class similar to unicode, but which make the difference between value and label
+    label is show when use as unicode, the .value attribute contain the actual value
+    Can be initialised with:
+        - basestring (label = value = given string)
+        - a tuple with (value, label)
+    XXX: comparaison is made against value, not the label which is the one displayed
+
+    """
+
+    def __new__(cls, option):
+        if (isinstance(option, cls)):
+            return option
+        elif isinstance(option, basestring):
+            value = label = option
+        elif (isinstance(option, tuple) and len(option) == 2):
+            value, label = option
+        else:
+            raise NotImplementedError
+        if not label:
+            label = value
+        instance = super(ListOption, cls).__new__(cls, label)
+        instance._value = value
+        return instance
+
+    def __eq__(self, other):
+        # XXX: try to compare values, if other has no value
+        #      (e.g. unicode string) try to compare to other itself
+        try:
+            return self._value == other._value
+        except AttributeError:
+            return self._value == other
+
+    def __ne__(self, other):
+        # XXX: see __eq__
+        try:
+            return self._value != other._value
+        except AttributeError:
+            return self._value != other
+
+    @property
+    def value(self):
+        """ return option value """
+        return self._value
+
+    @value.setter
+    def value(self, value):
+        self._value = value
+
+    @staticmethod
+    def fromOptions(options):
+        """ convert a list of string/tuple options to a list of listOption
+        @param options: list of managed option type (basestring, tuple)
+        return: list of ListOption
+        """
+        return [(ListOption(option)) for option in options]
+
+
 class GenericList(urwid.WidgetWrap):
     signals = ['click','change']
 
-    def __init__(self, options, style=[], align='left', option_type = SelectableText, on_click=None, on_change=None, user_data=None):
+    def __init__(self, options, style=None, align='left', option_type = SelectableText, on_click=None, on_change=None, user_data=None):
         """
         Widget managing list of string and their selection
         @param options: list of strings used for options
         @param style: list of string:
             - 'single' if only one must be selected
-            - 'no_first_select' nothing selected when list is first displayed 
+            - 'no_first_select' nothing selected when list is first displayed
             - 'can_select_none' if we can select nothing
         @param align: alignement of text inside the list
         @param on_click: method called when click signal is emited
         @param user_data: data sent to the callback for click signal
         """
+        if style is None:
+            style = []
         self.single = 'single' in style
         self.no_first_select = 'no_first_select' in style
         self.can_select_none = 'can_select_none' in style
         self.align = align
         self.option_type = option_type
         self.first_display = True
-        
+
         if on_click:
             urwid.connect_signal(self, 'click', on_click, user_data)
-        
+
         if on_change:
             urwid.connect_signal(self, 'change', on_change, user_data)
-        
+
         self.content = urwid.SimpleListWalker([])
         self.list_box = urwid.ListBox(self.content)
         urwid.WidgetWrap.__init__(self, self.list_box)
@@ -425,7 +474,8 @@ class GenericList(urwid.WidgetWrap):
         return self.list_box
 
     def changeValues(self, new_values):
-        """Change all value in one shot"""
+        """Change all values in one shot"""
+        new_values = ListOption.fromOptions(new_values)
         if not self.first_display:
             old_selected = self.getSelectedValues()
         widgets = []
@@ -445,7 +495,7 @@ class GenericList(urwid.WidgetWrap):
         display_widget = self.getDisplayWidget()
         self._set_w(display_widget)
         self._emit('change')
-        self.first_display = False 
+        self.first_display = False
 
     def selectValue(self, value):
         """Select the first item which has the given value"""
@@ -458,23 +508,35 @@ class GenericList(urwid.WidgetWrap):
                 return
             idx+=1
 
-class List(urwid.FlowWidget):
+
+class List(urwid.Widget):
     """FlowWidget list, same arguments as GenericList, with an additional one 'max_height'"""
     signals = ['click','change']
+    _sizing = frozenset(['flow'])
 
-    def __init__(self, options, style=[], max_height=5, align='left', option_type = SelectableText, on_click=None, on_change=None, user_data=None):
+    def __init__(self, options, style=None, max_height=5, align='left', option_type = SelectableText, on_click=None, on_change=None, user_data=None):
+        if style is None:
+            style = []
         self.genericList = GenericList(options, style, align, option_type, on_click, on_change, user_data)
-        self.max_height = max_height 
+        urwid.connect_signal(self.genericList, 'change', self._onChange)
+        urwid.connect_signal(self.genericList, 'click', self._onClick)
+        self.max_height = max_height
+
+    def _onChange(self, widget):
+        self._emit('change')
+
+    def _onClick(self, widget):
+        self._emit('click')
 
     def selectable(self):
         return True
 
     def keypress(self, size, key):
         return self.displayWidget(size,True).keypress(size, key)
-        
+
     def unselectAll(self, invisible=False):
         return self.genericList.unselectAll(invisible)
-    
+
     def deleteValue(self, value):
         return self.genericList.deleteValue(value)
 
@@ -495,15 +557,16 @@ class List(urwid.FlowWidget):
 
     def render(self, size, focus=False):
         return self.displayWidget(size, focus).render(size, focus)
-    
+
     def rows(self, size, focus=False):
         return self.displayWidget(size, focus).rows(size, focus)
 
     def displayWidget(self, size, focus):
         list_size = sum([wid.rows(size, focus) for wid in self.genericList.content])
-        height = min(list_size,self.max_height) or 1 
+        height = min(list_size,self.max_height) or 1
         return urwid.BoxAdapter(self.genericList, height)
 
+
 ## MISC ##
 
 class NotificationBar(urwid.WidgetWrap):
@@ -518,7 +581,7 @@ class NotificationBar(urwid.WidgetWrap):
         self.columns = urwid.Columns([('fixed',6,self.waitNotifs),self.message,('fixed',4,self.progress)])
         urwid.WidgetWrap.__init__(self, urwid.AttrMap(self.columns,'notifs'))
         self.notifs = []
-    
+
     def __modQueue(self):
         """must be called each time the notifications queue is changed"""
         self.waitNotifs.set_text(('notifs',"(%i)" % len(self.notifs) if self.notifs else ''))
@@ -586,7 +649,7 @@ class NotificationBar(urwid.WidgetWrap):
 class MenuBox(urwid.WidgetWrap):
     """Show menu items of a category in a box"""
     signals = ['click']
-    
+
     def __init__(self,parent,items):
         self.parent = parent
         self.selected = None
@@ -600,7 +663,7 @@ class MenuBox(urwid.WidgetWrap):
 
     def getValue(self):
         return self.selected
-   
+
     def keypress(self, size, key):
         if key=='up':
             if self.listBox.get_focus()[1] == 0:
@@ -609,7 +672,7 @@ class MenuBox(urwid.WidgetWrap):
             self.parent.keypress(size,'up')
             self.parent.keypress(size,key)
         return super(MenuBox,self).keypress(size,key)
-    
+
     def mouse_event(self, size, event, button, x, y, focus):
         if button == 3:
             self.parent.keypress(size,'up')
@@ -620,6 +683,7 @@ class MenuBox(urwid.WidgetWrap):
         self.selected = wid.getValue()
         self._emit('click')
 
+
 class Menu(urwid.WidgetWrap):
 
     def __init__(self,loop, x_orig=0):
@@ -635,21 +699,21 @@ class Menu(urwid.WidgetWrap):
         self.save_bottom = None
         col_rol = ColumnsRoller()
         urwid.WidgetWrap.__init__(self, urwid.AttrMap(col_rol,'menubar'))
-    
+
     def selectable(self):
         return True
 
     def getMenuSize(self):
         """return the current number of categories in this menu"""
         return len(self.menu_keys)
-   
+
     def setOrigX(self, orig_x):
         self.x_orig = orig_x
 
-    def __buildOverlay(self,menu_key,columns):
+    def __buildOverlay(self, menu_key, columns):
         """Build the overlay menu which show menuitems
         @param menu_key: name of the category
-        @colums: column number where the menubox must be displayed"""
+        @param columns: column number where the menubox must be displayed"""
         max_len = 0
         for item in self.menu[menu_key]:
             if len(item[0]) > max_len:
@@ -658,8 +722,8 @@ class Menu(urwid.WidgetWrap):
         self.save_bottom = self.loop.widget
         menu_box = MenuBox(self,[item[0] for item in self.menu[menu_key]])
         urwid.connect_signal(menu_box, 'click', self.onItemClick)
-        
-        self.loop.widget = urwid.Overlay(urwid.AttrMap(menu_box,'menubar'),self.save_bottom,('fixed left', columns),max_len+2,('fixed top',1),None) 
+
+        self.loop.widget = urwid.Overlay(urwid.AttrMap(menu_box,'menubar'),self.save_bottom,('fixed left', columns),max_len+2,('fixed top',1),None)
 
     def keypress(self, size, key):
         if key == 'down':
@@ -668,9 +732,9 @@ class Menu(urwid.WidgetWrap):
             if  self.save_bottom:
                 self.loop.widget = self.save_bottom
                 self.save_bottom = None
-            
+
         return self._w.base_widget.keypress(size, key)
-    
+
     def checkShortcuts(self, key):
         for shortcut in self.shortcuts.keys():
             if key == shortcut:
@@ -706,11 +770,11 @@ class Menu(urwid.WidgetWrap):
         if callback:
             self.keypress(None,'up')
             callback((category, item))
-    
+
     def onCategoryClick(self, button):
         self.__buildOverlay(button.get_label(),
-                            self.x_orig + self._w.base_widget.getStartCol(button)) 
-        
+                            self.x_orig + self._w.base_widget.getStartCol(button))
+
 
 class MenuRoller(urwid.WidgetWrap):
 
@@ -722,10 +786,10 @@ class MenuRoller(urwid.WidgetWrap):
         self.selected = 0
         self.name_list = []
         self.menus = {}
-             
-        self.columns = urwid.Columns([urwid.Text(''),urwid.Text('')]) 
+
+        self.columns = urwid.Columns([urwid.Text(''),urwid.Text('')])
         urwid.WidgetWrap.__init__(self, self.columns)
-        
+
         for menu_tuple in menus_list:
             name,menu = menu_tuple
             self.addMenu(name, menu)
@@ -733,13 +797,12 @@ class MenuRoller(urwid.WidgetWrap):
     def __showSelected(self):
         """show menu selected"""
         name_txt = u'\u21c9 '+self.name_list[self.selected]+u' \u21c7 '
-        current_name = ClickableText(name_txt) 
+        current_name = ClickableText(name_txt)
         name_len = len(name_txt)
         current_menu = self.menus[self.name_list[self.selected]]
         current_menu.setOrigX(name_len)
-        self.columns.widget_list[0] = current_name
-        self.columns.column_types[0]=('fixed', name_len)
-        self.columns.widget_list[1] = current_menu
+        self.columns.contents[0] = (current_name, ('given', name_len, False))
+        self.columns.contents[1] = (current_menu, ('weight', 1, False))
 
     def keypress(self, size, key):
         if key=='up':
@@ -752,7 +815,7 @@ class MenuRoller(urwid.WidgetWrap):
                 self.__showSelected()
         elif key=='right':
             if self.columns.get_focus_column()==0 and \
-                (isinstance(self.columns.widget_list[1], urwid.Text) or \
+                (isinstance(self.columns.contents[1][0], urwid.Text) or \
                 self.menus[self.name_list[self.selected]].getMenuSize()==0):
                 return #if we have no menu or the menu is empty, we don't go the right column
 
@@ -778,15 +841,17 @@ class MenuRoller(urwid.WidgetWrap):
         for menu in self.name_list:
             key = self.menus[menu].checkShortcuts(key)
         return key
-        
+
 
 ## DIALOGS ##
 
 class GenericDialog(urwid.WidgetWrap):
 
-    def __init__(self, widgets_lst, title, style=[], **kwargs):
+    def __init__(self, widgets_lst, title, style=None, **kwargs):
+        if style is None:
+            style = []
         frame_header = urwid.AttrMap(urwid.Text(title,'center'),'title')
-       
+
         buttons = None
 
         if "OK/CANCEL" in style:
@@ -811,32 +876,39 @@ class GenericDialog(urwid.WidgetWrap):
         urwid.WidgetWrap.__init__(self, decorated_frame)
 
 
-
 class InputDialog(GenericDialog):
     """Dialog with an edit box"""
 
-    def __init__(self, title, instrucions, style=['OK/CANCEL'], default_txt = '', **kwargs):
+    def __init__(self, title, instrucions, style=None, default_txt = '', **kwargs):
+        if style is None:
+            style = ['OK/CANCEL']
         instr_wid = urwid.Text(instrucions+':')
         edit_box = AdvancedEdit(edit_text=default_txt)
         GenericDialog.__init__(self, [instr_wid,edit_box], title, style, ok_value=edit_box, **kwargs)
         self._w.base_widget.set_focus('body')
 
+
 class ConfirmDialog(GenericDialog):
     """Dialog with buttons for confirm or cancel an action"""
 
-    def __init__(self, title, style=['YES/NO'], **kwargs):
+    def __init__(self, title, style=None, **kwargs):
+        if style is None:
+            style = ['YES/NO']
         GenericDialog.__init__(self, [], title, style, **kwargs)
 
+
 class Alert(GenericDialog):
     """Dialog with just a message and a OK button"""
 
     def __init__(self, title, message, style=['OK'], **kwargs):
         GenericDialog.__init__(self, [urwid.Text(message, 'center')], title, style, ok_value=None, **kwargs)
 
+
 ## CONTAINERS ##
 
-class ColumnsRoller(urwid.FlowWidget):
-    
+class ColumnsRoller(urwid.Widget):
+    _sizing = frozenset(['flow'])
+
     def __init__(self, widget_list = None, focus_column=0):
         self.widget_list = widget_list or []
         self.focus_column = focus_column
@@ -862,7 +934,7 @@ class ColumnsRoller(urwid.FlowWidget):
             return self.widget_list[self.focus_column][1].selectable()
         except IndexError:
             return False
-    
+
     def keypress(self, size, key):
         if key=='left':
             if self.focus_column>0:
@@ -895,7 +967,7 @@ class ColumnsRoller(urwid.FlowWidget):
         _prev = _next = False
         start_wid = 0
         end_wid = len(self.widget_list)-1
-        
+
         total_wid = sum([w[0] for w in self.widget_list])
         while total_wid > maxcol:
             if self.focus_column == end_wid:
@@ -910,11 +982,11 @@ class ColumnsRoller(urwid.FlowWidget):
                     _next = True
                 total_wid-=self.widget_list[end_wid][0]
                 end_wid-=1
-        
+
         cols_left = maxcol - total_wid
         self.__start = start_wid #we need to keep it for getStartCol
         return _prev,_next,start_wid,end_wid,cols_left
-        
+
 
     def mouse_event(self, size, event, button, x, y, focus):
         (maxcol,)=size
@@ -927,7 +999,7 @@ class ColumnsRoller(urwid.FlowWidget):
             if x==maxcol-1 and _next:
                 self.keypress(size,'right')
                 return True
-            
+
             current_pos = 1 if _prev else 0
             idx = 0
             while current_pos<x and idx<len(self.widget_list):
@@ -937,20 +1009,20 @@ class ColumnsRoller(urwid.FlowWidget):
                     self._invalidate()
                     if not hasattr(widget,'mouse_event'):
                         return False
-                    return widget.mouse_event((width,0), event, button, 
+                    return widget.mouse_event((width,0), event, button,
                         x-current_pos, 0, focus)
 
                 current_pos+=self.widget_list[idx][0]
                 idx+=1
-        
+
         return False
-    
+
     def render(self, size, focus=False):
         if not self.widget_list:
             return SolidCanvas(" ", size[0], 1)
 
         _prev,_next,start_wid,end_wid,cols_left = self.__calculate_limits(size)
-        
+
         idx=start_wid
         render = []
 
@@ -975,7 +1047,7 @@ class FocusFrame(urwid.Frame):
         ret = urwid.Frame.keypress(self, size, key)
         if not ret:
             return
-        
+
         if key == 'tab':
             focus_list = ('header','body','footer')
             focus_idx = focus_list.index(self.focus_part)
@@ -986,7 +1058,8 @@ class FocusFrame(urwid.Frame):
                 if widget!=None and widget.selectable():
                     self.set_focus(focus_name)
 
-        return ret 
+        return ret
+
 
 class TabsContainer(urwid.WidgetWrap):
     signals = ['click']
@@ -1034,11 +1107,13 @@ class TabsContainer(urwid.WidgetWrap):
             self._buttons_cont.set_focus(0)
             self.__buttonClicked(button,True)
 
-    def addTab(self,name,content=[]):
+    def addTab(self,name,content=None):
         """Add a page to the container
         @param name: name of the page (what appear on the tab)
         @param content: content of the page
         @return: ListBox (content of the page)"""
+        if content is None:
+            content = []
         listbox = urwid.ListBox(urwid.SimpleListWalker(content))
         self.tabs.append([name,listbox])
         self.__appendButton(name)
@@ -1048,7 +1123,170 @@ class TabsContainer(urwid.WidgetWrap):
         """Add a widget on the bottom of the tab (will be displayed on all pages)
         @param widget: FlowWidget"""
         self._w.footer = widget
-    
+
+
+class HighlightColumns(urwid.AttrMap):
+    """ Decorated columns which highlight all or some columns """
+
+    def __init__(self, highlight_cols, highlight_attr, *args, **kwargs):
+        """ Create the HighlightColumns
+        @param highlight_cols: tuple of columns to highlight, () to highlight to whole row
+        @param highlight_attr: name of the attribute to use when focused
+        other parameter are passed to urwid Columns
+
+        """
+        columns = urwid.Columns(*args, **kwargs)
+        self.highlight_cols = highlight_cols
+        self.highlight_attr = highlight_attr
+        self.has_focus = False
+        if highlight_cols == ():
+            super(HighlightColumns, self).__init__(columns, None, highlight_attr)
+            self.highlight_cols = None
+        else:
+            super(HighlightColumns, self).__init__(columns, None)
+
+    @property
+    def options(self):
+        return self.base_widget.options
+
+    @property
+    def contents(self):
+        return self.base_widget.contents
+
+    @property
+    def focus_position(self):
+        return self.base_widget.focus_position
+
+    @focus_position.setter
+    def focus_position(self, value):
+        self.base_widget.focus_position = value
+
+    def addWidget(self, wid, options):
+        """ Add a widget to the columns
+        Widget is wrapped with AttrMap, that's why Columns.contents should not be used directly for appending new widgets
+        @param wid: widget to add
+        @param options: result of Columns.options(...)
+
+        """
+        wrapper = urwid.AttrMap(wid, None)
+        self.base_widget.contents.append((wrapper, options))
+
+
+    def render(self, size, focus=False):
+        if self.highlight_cols and focus != self.has_focus:
+            self.has_focus = focus
+            for idx in self.highlight_cols:
+                wid = self.base_widget.contents[idx][0]
+                wid.set_attr_map({None: self.highlight_attr if focus else None})
+
+        return super(HighlightColumns, self).render(size, focus)
+
+
+class TableContainer(urwid.WidgetWrap):
+    """ Widgets are disposed in row and columns """
+    signals = ['click']
+
+    def __init__(self, items=None, columns=None, dividechars=1, row_selectable=False, select_key='enter', options=None):
+        """ Create a TableContainer
+        @param items: iterable of widgets to add to this container
+        @param columns: nb of columns of this table
+        @param dividechars: same as dividechars param for urwid.Columns
+        @param row_selectable: if True, row are always selectable, even if they don't contain any selectable widget
+        @param options: dictionnary with the following keys:
+            - ADAPT: tuple of columns for which the size must be adapted to its contents,
+                     empty tuple for all columns
+            - HIGHLIGHT: tuple of columns which must be higlighted on focus,
+                         empty tuple for the whole row
+            - FOCUS_ATTR: Attribute name to use when focused (see HIGHLIGHT). Default is "table_selected"
+
+        """
+        pile = urwid.Pile([])
+        super(TableContainer, self).__init__(pile)
+        if items is None:
+            items = []
+        if columns is None: # if columns is None, we suppose only one row is given in items
+            columns = len(items)
+        assert columns
+        self._columns = columns
+        self._row_selectable = row_selectable
+        self.select_key = select_key
+        if options is None:
+            options = {}
+        for opt in ['ADAPT', 'HIGHLIGHT']:
+            if opt in options:
+                try:
+                    options[opt] = tuple(options[opt])
+                except TypeError:
+                    warning('[%s] option is not a tuple' % opt)
+                    options[opt] = ()
+        self._options = options
+        self._dividechars = dividechars
+        self._idx = 0
+        self._longuest = self._columns * [0]
+        self._next_row_idx = None
+        for item in items:
+            self.addWidget(item)
+
+    def _getIdealSize(self, widget):
+        """ return preferred size for widget, or 0 if we can't find it """
+        try:
+            return len(widget.text)
+        except AttributeError:
+            return 0
+
+    def keypress(self, size, key):
+        if key == self.select_key and self._row_selectable:
+            self._emit('click')
+        else:
+            return super(TableContainer, self).keypress(size, key)
+
+
+    def addWidget(self, widget):
+        # TODO: use a contents property ?
+        pile = self._w
+        col_idx = self._idx % self._columns
+
+        options = None
+
+        if col_idx == 0:
+            # we have a new row
+            columns = HighlightColumns(self._options.get('HIGHLIGHT'), self._options.get('FOCUS_ATTR', 'table_selected'), [], dividechars=self._dividechars)
+            columns.row_idx = self._next_row_idx
+            pile.contents.append((columns, pile.options()))
+        else:
+            columns = pile.contents[-1][0]
+
+        if 'ADAPT' in self._options and (col_idx in self._options['ADAPT']
+            or self._options['ADAPT'] == ()):
+            current_len = self._getIdealSize(widget)
+            longuest = self._longuest[col_idx]
+            max_len = max(longuest, current_len)
+            if max_len > longuest:
+                self._longuest[col_idx] = max_len
+                for wid,_ in pile.contents[:-1]:
+                    col = wid.base_widget
+                    col.contents[col_idx] = (col.contents[col_idx][0], col.options('given', max_len))
+            options = columns.options('given', max_len) if max_len else columns.options()
+
+        columns.addWidget(widget, options or columns.options())
+
+        if self._row_selectable and col_idx == self._columns - 1:
+            columns.addWidget(urwid.SelectableIcon(''), columns.options('given', 0))
+
+        if not columns.selectable() and columns.contents[-1][0].base_widget.selectable():
+            columns.focus_position = len(columns.contents)-1
+        self._idx += 1
+
+    def setRowIndex(self, idx):
+        self._next_row_idx = idx
+
+    def getSelectedWidgets(self):
+        columns = self._w.focus
+        return (wid for wid, _ in columns.contents)
+
+    def getSelectedIndex(self):
+        columns = self._w.focus
+        return columns.row_idx
 
 ## DECORATORS ##
 class LabelLine(urwid.LineBox):
@@ -1059,10 +1297,11 @@ class LabelLine(urwid.LineBox):
         top_columns = self._w.widget_list[0]
         top_columns.widget_list[1] = label_widget
 
+
 class VerticalSeparator(urwid.WidgetDecoration, urwid.WidgetWrap):
     def __init__(self, original_widget, left_char = u"│", right_char = ''):
         """Draw a separator on left and/or right of original_widget."""
-        
+
         widgets = [original_widget]
         if left_char:
             widgets.insert(0, ('fixed', 1, urwid.SolidFill(left_char)))
@@ -1072,4 +1311,4 @@ class VerticalSeparator(urwid.WidgetDecoration, urwid.WidgetWrap):
         urwid.WidgetDecoration.__init__(self, original_widget)
         urwid.WidgetWrap.__init__(self, columns)
 
-        
+
